
CREATE OR REPLACE LIBRARY mail_utl_2_l
AS
   '/oracle/local/lib/libmail_utl_2.so';

CREATE OR REPLACE 
PACKAGE mail_utl
IS
    FUNCTION send_mail
       (return_address  IN CHAR,
        recipient_list  IN CHAR,
        subject_line    IN CHAR,
        message_text    IN CHAR)
    RETURN PLS_INTEGER;
    PRAGMA RESTRICT_REFERENCES (send_mail, WNDS, RNDS, WNPS, RNPS);

END mail_utl;
/

CREATE OR REPLACE
PACKAGE BODY mail_utl
IS
    FUNCTION send_mail
       (return_address  IN CHAR,
        recipient_list  IN CHAR,
        subject_line    IN CHAR,
        message_text    IN CHAR)
    RETURN PLS_INTEGER
    IS EXTERNAL
       LIBRARY mail_utl_2_l
       NAME "send_mail"
       LANGUAGE C
       PARAMETERS
          (return_address STRING,
           recipient_list STRING,
           subject_line   STRING,
           message_text   STRING,
           RETURN sb4);
END mail_utl;
/


SET SERVEROUTPUT ON SIZE 1000000

DECLARE
   exit_status  PLS_INTEGER;
   eol          CHAR      := CHR(10);
   message      CHAR(80)  := 'line 1' || eol ||
                             'line 2' || eol ||
                             'line 3' || eol;
BEGIN
   /* Send a multi-line message from user "fred" to users "bill" and "fred" */
   exit_status := mail_utl.send_mail ('fred',
                                      'bill, fred',
                                      'Mail from PL/SQL!',
                                      message);

   DBMS_OUTPUT.PUT_LINE ('send_mail exit_status = ' || exit_status);

   /* Since no indicator variables are passed in, we must specify all
   || parameters and they cannot be NULL.  The following attempt to call
   || the external procedure will fail because NULL is passed as the
   || "subject_line" parameter.
   || The oracle error will be: "ORA-01405: fetched column value is NULL"
   */
   exit_status := mail_utl.send_mail ('fred',
                                      'bill, fred',
                                      NULL,
                                      message);

   DBMS_OUTPUT.PUT_LINE ('send_mail exit_status = ' || exit_status);
END;
/
