/****************************************************************************
** File:    random_utl_1.c
**
** Description: This module contains a simple wrapper function for the
**              XPG4 standard library function nrand48() so that it can
**              be called as an external PL/SQL function from Oracle8.
**
**              The nrand48() function is one of a group of functions that
**              generate pseudo-random numbers using the linear congruential
**              algorithm and 48-bit integer arithmetic. Function nrand48()
**              returns non-negative integers uniformly distributed over
**              the interval 0 .. 2147483648.
**
**              The nrand48() function is useful because it is "virtually"
**              stateless. The last 48-bit value computed by the linear
**              congruential algorithm is provided by the caller.  So, this
**              function can easily be used to implement independent
**              pseudo-random number objects each with its own independent
**              series of pseudo-random values. 
**
**      Author: Fred Polizo, Jr.    FP Systems
****************************************************************************/

#include <stdlib.h>
#include <oratypes.h>

#ifdef _POSIX_C_SOURCE
extern long nrand48(unsigned short xsubi[3]);
#endif

/****************************************************************************
** Function:    plsql_nrand48
**
** Description: A wrapper for the nrand48() pseudo-random number generator.
**              The caller provides the last 48-bit value computed by
**              nrand48() as three unsigned 16-bit values.  This will be used
**              to compute the next 48-bit value in the given pseudo-random
**              number sequence.
**
**              Upon return, the 16-bit values will be updated to reflect the
**              newly computed 48-bit value and the function return value
**              will be the next pseudo-random number.
****************************************************************************/
sb4 
plsql_nrand48 (ub2 *pState0, ub2 *pState1, ub2 *pState2)
{
    sb4 pseudoRandomValue;
    ub2 stateArray[3];

    /* format state values into the array expected by nrand48() */

    stateArray[0] = *pState0;
    stateArray[1] = *pState1;
    stateArray[2] = *pState2;

    /* generate the next pseudo-random value */
    
    pseudoRandomValue = nrand48(stateArray);

    /* return the new state values to the caller */

    *pState0 = stateArray[0];
    *pState1 = stateArray[1];
    *pState2 = stateArray[2];

    return pseudoRandomValue;
}