
CREATE OR REPLACE LIBRARY timezone_utl_l
AS
   '/oracle/local/lib/libtz_utl.so';

CREATE OR REPLACE 
PACKAGE timezone_utl
IS
    PROCEDURE timezone
       (local_timezone OUT VARCHAR2);
    PRAGMA RESTRICT_REFERENCES (timezone, WNDS, RNDS, WNPS, RNPS);

END timezone_utl;
/

CREATE OR REPLACE
PACKAGE BODY timezone_utl
IS
    PROCEDURE timezone
       (local_timezone OUT VARCHAR2)
    IS EXTERNAL
       LIBRARY timezone_utl_l
       NAME "get_timezone"
       LANGUAGE C
       PARAMETERS
          (local_timezone STRING,
           local_timezone MAXLEN sb4,
           local_timezone INDICATOR sb2);
END timezone_utl;
/


SET SERVEROUTPUT ON SIZE 1000000

DECLARE
   local_time   DATE;
   local_tz     VARCHAR2(3);
   target_time  DATE;
   target_tz    VARCHAR2(3) := 'GMT';
BEGIN
   /* Call the external procedure to discover the local timezone */
   timezone_utl.timezone (local_tz);

   IF ( local_tz IS NOT NULL ) THEN
      DBMS_OUTPUT.PUT_LINE ('timezone returned = "' || local_tz || '"');

      /* Convert the current time to the target timezone */
      local_time  := SYSDATE;
      target_time := NEW_TIME (local_time, local_tz, target_tz);

      /* Display the original and converted times with their timezones */
      DBMS_OUTPUT.PUT_LINE (TO_CHAR (local_time,  'DD-MON-YYYY HH24:MI:SS')
                            || ' ' || local_tz );
      DBMS_OUTPUT.PUT_LINE ('=');
      DBMS_OUTPUT.PUT_LINE (TO_CHAR (target_time, 'DD-MON-YYYY HH24:MI:SS')
                            || ' ' || target_tz );
   ELSE
      DBMS_OUTPUT.PUT_LINE ('local_tz returned NULL');
   END IF;

END;
/